package com.thuwater.data.client;

import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.thuwater.data.client.history.HistoryService;
import com.thuwater.data.client.model.Config;
import com.thuwater.data.client.model.ThData;
import com.thuwater.data.client.pull.PullService;
import com.thuwater.data.client.push.PushHttpServer;
import com.thuwater.data.client.service.DataHandler;
import com.thuwater.data.client.service.DataHandlerImpl;
import com.thuwater.data.client.service.TokenService;

/**
 * @Author zhangpeng
 * @Date 2019/12/11 10:35
 */
public class App {
    private static final Logger log = LoggerFactory.getLogger(App.class);

    private static ExecutorService es = Executors.newFixedThreadPool(1);


    public static void main(String[] args) {
        Config config = Config.instance();
        TokenService ts = new TokenService();
        DataHandler dh = new DataHandlerImpl();

        switch (config.getMode()) {
            case push: { // 推送模式情况
                PushHttpServer pushServer = new PushHttpServer(config.getPort(), config.getUri(), ts, dh);
                es.submit(pushServer::start);
                log.info("推送模式开始");
                break;
            }
            case pull: { // 拉取模式情况
                PullService pullService = new PullService(ts, dh);
                es.submit(pullService::start);
                log.info("拉取模式开始");
                break;
            }
            case history: { // 历史数据模式情况
                HistoryService historyService = new HistoryService(ts);
                // 这里使用的是示例时间与 placeId ，请根据实际情况修改 TODO
                List<ThData> datas = historyService.getHistoryData(
                        "6252",
                        LocalDateTime.of(2019, 10, 20, 0, 0, 0),
                        LocalDateTime.of(2019, 10, 20, 3, 0, 0)
                );
                log.info("历史数据模式开始");
                dh.handleDatas(datas);
                break;
            }
            default:
                log.error("必须设置对接模式");
        }

    }
}
