package com.thuwater.data.client.history;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.type.TypeReference;
import com.thuwater.data.client.model.ThData;
import com.thuwater.data.client.service.TokenService;
import com.thuwater.data.client.utils.DateTimeUtil;
import com.thuwater.data.client.utils.HttpClient;
import com.thuwater.data.client.utils.JsonUtil;

import okhttp3.Call;
import okhttp3.HttpUrl;
import okhttp3.Request;
import okhttp3.Response;

/**
 * @Author zhangpeng
 * @Date 2019/12/27 14:56
 */
public class HistoryService {
    private static final Logger log = LoggerFactory.getLogger(HistoryService.class);
    private static final TypeReference<List<ThData>> typeReference = new TypeReference<List<ThData>>() {};
    private final TokenService tokenService;

    public HistoryService(TokenService tokenService) {
        this.tokenService = tokenService;
    }

    @SuppressWarnings("ConstantConditions")
    public List<ThData> getHistoryData(String placeId, LocalDateTime st, LocalDateTime et) {
        HttpUrl url = HttpUrl.parse("https://dev.thuwater.com/api/data/range")
                             .newBuilder()
                             .addQueryParameter("placeId", placeId)
                             .addQueryParameter("st", DateTimeUtil.dateTimeToString(st))
                             .addQueryParameter("et", DateTimeUtil.dateTimeToString(et))
                             .build();
        Request request = new Request.Builder()
                .header("X-TH-TOKEN", tokenService.genToken())
                .url(url).get().build();
        Call call = HttpClient.instance().newCall(request);
        try (Response res = call.execute()) {
            if (res.isSuccessful()) {
                return JsonUtil.toObject(res.body().string(), typeReference);
            }
        } catch (IOException e) {
            log.error("IO异常", e);
        } catch (Exception e) {
            log.error("异常", e);
        }
        return Collections.emptyList();
    }
}
