package com.thuwater.data.client.pull;

import java.io.IOException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.type.TypeReference;
import com.thuwater.data.client.model.ThData;
import com.thuwater.data.client.service.DataHandler;
import com.thuwater.data.client.service.TokenService;
import com.thuwater.data.client.utils.HttpClient;
import com.thuwater.data.client.utils.JsonUtil;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;

/**
 * @Author zhangpeng
 * @Date 2019/12/27 11:40
 */
public class PullService {
    private static final Logger log = LoggerFactory.getLogger(PullService.class);
    private static final TypeReference<List<ThData>> typeReference = new TypeReference<List<ThData>>() {};

    private final TokenService tokenService;
    private final DataHandler dataHandler;


    public PullService(TokenService tokenService, DataHandler dataHandler) {
        this.tokenService = tokenService;
        this.dataHandler = dataHandler;
    }

    public void start() {
        while (true) {
            OkHttpClient client = HttpClient.instance();
            Request request = new Request.Builder()
                    .url("https://dev.thuwater.com/api/data/realtime")
                    .get()
                    .addHeader("X-TH-TOKEN", tokenService.genToken())
                    .addHeader("Host", "dev.thuwater.com")
                    .build();
            long sleepTime = 0;
            try (Response response = client.newCall(request).execute()) {
                if (response.isSuccessful()) {
                    ResponseBody body = response.body();
                    if (body == null) {
                        // 当返回为空的时候说明已经没有最新数据等待同步，建议等待5分钟后再请求
                        sleepTime = 5 * 60 * 1000;
                        continue;
                    }
                    List<ThData> thDataList = JsonUtil.toObject(body.string(), typeReference);
                    if (thDataList.size() == 0) {
                        // 当返回为空的时候说明已经没有最新数据等待同步，建议等待5分钟后再请求
                        sleepTime = 5 * 60 * 1000;
                    } else if (thDataList.size() < 100) {
                        dataHandler.handleDatas(thDataList);
                        // 当返回数据少于100条的时候，建议等待1分钟后再请求
                        sleepTime = 60 * 1000;
                    } else {
                        // 当返回数据大于100条的时候，建议不等待直接继续请求最新数据
                        dataHandler.handleDatas(thDataList);
                    }
                } else {
                    log.error("请求返回错误，请检查[{}]", response);
                    sleep(60 * 1000);// 过1分钟后再重试
                    if (response.code() == 401) {
                        tokenService.genToken(true);
                    }
                }
            } catch (IOException e) {
                log.error("IO错误", e);
                sleep(60 * 1000);// 过1分钟后再重试
            } catch (Exception e) {
                log.error("其他错误", e);
                sleep(60 * 1000);// 过1分钟后再重试
            } finally {
                sleep(sleepTime);
            }
        }
    }

    private void sleep(long time) {
        try {
            log.debug("sleep for " + time);
            Thread.sleep(time);
        } catch (InterruptedException ex) {
            log.warn("线程退出", ex);
        }
    }
}
