package com.thuwater.data.client.push;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.thuwater.data.client.service.DataHandler;
import com.thuwater.data.client.service.TokenService;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.HttpServerExpectContinueHandler;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;

/**
 * @Author zhangpeng
 * @Date 2019/12/11 11:23
 */
public class PushHttpServer {
    private static final Logger log = LoggerFactory.getLogger(PushHttpServer.class);
    private final TokenService tokenService;
    private final DataHandler dataHandler;
    private final int port;
    private final String uri;

    public PushHttpServer(int port, String uri, TokenService tokenService, DataHandler dataHandler) {
        this.port = port;
        this.uri = uri;
        this.tokenService = tokenService;
        this.dataHandler = dataHandler;
    }

    public void start() {
        // Configure the server.
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        try {
            ServerBootstrap b = new ServerBootstrap();
            b.group(bossGroup, workerGroup)
             .channel(NioServerSocketChannel.class)
             //保持连接数
             .option(ChannelOption.SO_BACKLOG, 300)
             //有数据立即发送
             .option(ChannelOption.TCP_NODELAY, true)
             //保持连接
             .childOption(ChannelOption.SO_KEEPALIVE, true)
             .handler(new LoggingHandler(LogLevel.INFO))
             .childHandler(new ChannelInitializer<SocketChannel>() {
                 @Override
                 protected void initChannel(SocketChannel ch) {
                     ChannelPipeline p = ch.pipeline();
                     /**
                      * 或者使用HttpRequestDecoder & HttpResponseEncoder
                      */
                     p.addLast(new HttpServerCodec());
                     /**
                      * 在处理POST消息体时需要加上
                      */
                     p.addLast(new HttpObjectAggregator(1024 * 1024));
                     p.addLast(new HttpServerExpectContinueHandler());
                     p.addLast(new HttpServerHandler(uri, tokenService, dataHandler));
                 }
             });

            Channel ch = b.bind(port).sync().channel();
            log.info("Netty http server listening on port " + port);
            ch.closeFuture().sync();
        } catch (InterruptedException e) {
            log.warn("Interrupted:", e);
        } finally {
            log.warn("Netty http server is stopping");
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
            log.warn("Netty http server stopped");
        }
    }
}
