package com.thuwater.data.client.service;

import java.time.LocalDateTime;
import java.util.Map;

import javax.annotation.Nullable;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.thuwater.data.client.model.Config;
import com.thuwater.data.client.utils.DateTimeUtil;
import com.thuwater.data.client.utils.HttpClient;
import com.thuwater.data.client.utils.JsonUtil;

import okhttp3.*;

/**
 * @Author zhangpeng
 * @Date 2019/12/11 14:13
 */
public class TokenService {
    private static final Logger log = LoggerFactory.getLogger(TokenService.class);
    private final String theWhKey = Config.instance().getWhkey();
    private volatile String token;
    private volatile LocalDateTime expireTime = LocalDateTime.now().minusYears(1);
    private RequestBody tokenFreshbody = new FormBody.Builder()
            .add("username", Config.instance().getUsername())
            .add("password", Config.instance().getPassword())
            .build();
    private Request tokenFreshRequest = new Request.Builder()
            .url("https://dev.thuwater.com/api/permission/token")
            .post(tokenFreshbody)
            .build();
    private final Object lock = new Object();

    public TokenService() {
        init();
    }

    private void init() {
        if (!freshToken(true, 3, true)) {
            String msg = "初始化获取token失败";
            log.error(msg);
            throw new IllegalStateException(msg);
        }
    }

    private boolean freshToken(final boolean retry, final int maxRetry, boolean forceFresh) {
        if (!forceFresh && expireTime.isAfter(LocalDateTime.now())) {
            return true;
        }
        synchronized (lock) {
            if (expireTime.isAfter(LocalDateTime.now())) {
                return true;
            }
            OkHttpClient client = HttpClient.instance();
            int retryCount = 0;
            do {
                retryCount++;
                final Call call = client.newCall(tokenFreshRequest);
                try (Response response = call.execute()) {
                    if (response.isSuccessful() && response.body() != null) {
                        Map<String, String> tokenMap = JsonUtil.toMapFromBytes(response.body().bytes());
                        this.token = tokenMap.get("token");
                        this.expireTime = DateTimeUtil.format(tokenMap.get("validDate"));
                        return true;
                    } else {
                        log.error(response.toString());
                    }
                } catch (Exception e) {
                    log.error("重试第" + retryCount + "次", e);
                    try {
                        Thread.sleep(30000L);
                    } catch (InterruptedException ignore) {
                    }
                }
            } while (retry && retryCount <= maxRetry + 1);
        }
        return false;
    }

    public boolean validWhKey(@Nullable String thisWhKey) {
        if (StringUtils.isBlank(theWhKey) && StringUtils.isNotBlank(thisWhKey)) return false;
        if (StringUtils.isBlank(theWhKey) && StringUtils.isBlank(thisWhKey)) return true;
        return this.theWhKey.equals(thisWhKey);
    }

    public String genToken() {
        if (expireTime.isBefore(LocalDateTime.now())) {
            freshToken(true, 3, true);
        }
        return token;
    }

    public String genToken(boolean forceFresh) {
        if (forceFresh) {
            freshToken(true, 3, forceFresh);
        }
        return genToken();
    }
}
